@@ -1141,6 +1141,10 @@ class OpenAIStreamingClient(OpenAIBaseStreamingClient):
 
 
 _OPENAI_REASONING_MODELS = [
+    "gpt-5",
+    "gpt-5-mini",
+    "gpt-5-nano",
+    "gpt-5-chat-latest",
     "o1",
     "o1-pro",
     "o1-2024-12-17",
@@ -1201,50 +1205,6 @@ class OpenAIReasoningNonStreamingClient(
     OpenAIReasoningReasoningModelsMixin,
     OpenAIStreamingClient,
 ):
-    @override
-    async def chat_completion_create(
-        self,
-        messages: list[
-            tuple[ChatCompletionMessageRole, str, Optional[str], Optional[list[JSONScalarType]]]
-        ],
-        tools: list[JSONScalarType],
-        **invocation_parameters: Any,
-    ) -> AsyncIterator[ChatCompletionChunk]:
-        from openai import NOT_GIVEN
-
-        # Convert standard messages to OpenAI messages
-        openai_messages = []
-        for message in messages:
-            openai_message = self.to_openai_chat_completion_param(*message)
-            if openai_message is not None:
-                openai_messages.append(openai_message)
-
-        throttled_create = self.rate_limiter._alimit(self.client.chat.completions.create)
-        response = await throttled_create(
-            messages=openai_messages,
-            model=self.model_name,
-            stream=False,
-            tools=tools or NOT_GIVEN,
-            **invocation_parameters,
-        )
-
-        if response.usage is not None:
-            self._attributes.update(dict(self._llm_token_counts(response.usage)))
-
-        choice = response.choices[0]
-        if choice.message.content:
-            yield TextChunk(content=choice.message.content)
-
-        if choice.message.tool_calls:
-            for tool_call in choice.message.tool_calls:
-                yield ToolCallChunk(
-                    id=tool_call.id,
-                    function=FunctionCallChunk(
-                        name=tool_call.function.name,
-                        arguments=tool_call.function.arguments,
-                    ),
-                )
-
     def to_openai_chat_completion_param(
         self,
         role: ChatCompletionMessageRole,
@@ -1677,6 +1637,8 @@ class AnthropicStreamingClient(PlaygroundStreamingClient):
     model_names=[
         "claude-sonnet-4-0",
         "claude-sonnet-4-20250514",
+        "claude-opus-4-1",
+        "claude-opus-4-1-20250805",
         "claude-opus-4-0",
         "claude-opus-4-20250514",
         "claude-3-7-sonnet-latest",
